
#ifndef VTKEMBOSSINGREPRESENTATIONS_EXPORT_H
#define VTKEMBOSSINGREPRESENTATIONS_EXPORT_H

#ifdef VTKEMBOSSINGREPRESENTATIONS_STATIC_DEFINE
#  define VTKEMBOSSINGREPRESENTATIONS_EXPORT
#  define VTKEMBOSSINGREPRESENTATIONS_NO_EXPORT
#else
#  ifndef VTKEMBOSSINGREPRESENTATIONS_EXPORT
#    ifdef vtkEmbossingRepresentations_EXPORTS
        /* We are building this library */
#      define VTKEMBOSSINGREPRESENTATIONS_EXPORT __declspec(dllexport)
#    else
        /* We are using this library */
#      define VTKEMBOSSINGREPRESENTATIONS_EXPORT __declspec(dllimport)
#    endif
#  endif

#  ifndef VTKEMBOSSINGREPRESENTATIONS_NO_EXPORT
#    define VTKEMBOSSINGREPRESENTATIONS_NO_EXPORT 
#  endif
#endif

#ifndef VTKEMBOSSINGREPRESENTATIONS_DEPRECATED
#  define VTKEMBOSSINGREPRESENTATIONS_DEPRECATED __declspec(deprecated)
#endif

#ifndef VTKEMBOSSINGREPRESENTATIONS_DEPRECATED_EXPORT
#  define VTKEMBOSSINGREPRESENTATIONS_DEPRECATED_EXPORT VTKEMBOSSINGREPRESENTATIONS_EXPORT VTKEMBOSSINGREPRESENTATIONS_DEPRECATED
#endif

#ifndef VTKEMBOSSINGREPRESENTATIONS_DEPRECATED_NO_EXPORT
#  define VTKEMBOSSINGREPRESENTATIONS_DEPRECATED_NO_EXPORT VTKEMBOSSINGREPRESENTATIONS_NO_EXPORT VTKEMBOSSINGREPRESENTATIONS_DEPRECATED
#endif

#if 0 /* DEFINE_NO_DEPRECATED */
#  ifndef VTKEMBOSSINGREPRESENTATIONS_NO_DEPRECATED
#    define VTKEMBOSSINGREPRESENTATIONS_NO_DEPRECATED
#  endif
#endif

#endif /* VTKEMBOSSINGREPRESENTATIONS_EXPORT_H */
