
#ifndef VTKSTREAMINGPARTICLES_EXPORT_H
#define VTKSTREAMINGPARTICLES_EXPORT_H

#ifdef VTKSTREAMINGPARTICLES_STATIC_DEFINE
#  define VTKSTREAMINGPARTICLES_EXPORT
#  define VTKSTREAMINGPARTICLES_NO_EXPORT
#else
#  ifndef VTKSTREAMINGPARTICLES_EXPORT
#    ifdef vtkStreamingParticles_EXPORTS
        /* We are building this library */
#      define VTKSTREAMINGPARTICLES_EXPORT __declspec(dllexport)
#    else
        /* We are using this library */
#      define VTKSTREAMINGPARTICLES_EXPORT __declspec(dllimport)
#    endif
#  endif

#  ifndef VTKSTREAMINGPARTICLES_NO_EXPORT
#    define VTKSTREAMINGPARTICLES_NO_EXPORT 
#  endif
#endif

#ifndef VTKSTREAMINGPARTICLES_DEPRECATED
#  define VTKSTREAMINGPARTICLES_DEPRECATED __declspec(deprecated)
#endif

#ifndef VTKSTREAMINGPARTICLES_DEPRECATED_EXPORT
#  define VTKSTREAMINGPARTICLES_DEPRECATED_EXPORT VTKSTREAMINGPARTICLES_EXPORT VTKSTREAMINGPARTICLES_DEPRECATED
#endif

#ifndef VTKSTREAMINGPARTICLES_DEPRECATED_NO_EXPORT
#  define VTKSTREAMINGPARTICLES_DEPRECATED_NO_EXPORT VTKSTREAMINGPARTICLES_NO_EXPORT VTKSTREAMINGPARTICLES_DEPRECATED
#endif

#if 0 /* DEFINE_NO_DEPRECATED */
#  ifndef VTKSTREAMINGPARTICLES_NO_DEPRECATED
#    define VTKSTREAMINGPARTICLES_NO_DEPRECATED
#  endif
#endif

#endif /* VTKSTREAMINGPARTICLES_EXPORT_H */
