/*=========================================================================

  Program:   Visualization Toolkit
  Module:    vtk_xdmf2.h

  Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
  All rights reserved.
  See Copyright.txt or http://www.kitware.com/Copyright.htm for details.

     This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE.  See the above copyright notice for more information.

=========================================================================*/
#ifndef vtk_xdmf2_h
#define vtk_xdmf2_h

/* Use the xdmf2 library configured for VTK.  */
#define VTK_MODULE_USE_EXTERNAL_vtkxdmf2 0

#if VTK_MODULE_USE_EXTERNAL_vtkxdmf2
#else
#define VTKXDMF2_HEADER(x)  <vtkxdmf2/libsrc/x>
#endif

#endif
